import * as i0 from '@angular/core';
import { Injectable, Optional, NgModule, SkipSelf } from '@angular/core';
import { of } from 'rxjs';
import moment from 'moment';
import * as i2 from '@angular/common/http';
import { CommonModule } from '@angular/common';

class ServiceConfig {
}

const defaultCachingConfig = {
    enabled: false,
    lifeMinutes: 720
};

const defaultLoggingConfig = {
    dataCached: false,
    cachedDataReturned: false,
    freshDataReturned: false,
    cacheCleared: false
};

const defaultConfig = {
    apiUrl: 'http://utilityapi/users',
    caching: defaultCachingConfig,
    logging: defaultLoggingConfig
};

//#region Imports
//#endregion
class ActiveDirectoryService {
    //#endregion
    //#region Initialization
    // If a ServiceConfig object is not provided when the module is imported, then default values are pulled in from default-config.ts.
    constructor(config, http) {
        var _a, _b, _c;
        this.config = config;
        this.http = http;
        this.cache = null;
        this.defaultCacheLifeMinutes = 720;
        this.config = (_a = this.config) !== null && _a !== void 0 ? _a : defaultConfig;
        this.config.caching = (_b = this.config.caching) !== null && _b !== void 0 ? _b : defaultCachingConfig;
        this.config.logging = (_c = this.config.logging) !== null && _c !== void 0 ? _c : defaultLoggingConfig;
    }
    //#region Properties
    get cacheExpired() {
        var _a, _b;
        if (!this.cache) {
            return true;
        }
        const lifeMinutes = (_b = (_a = this.config.caching) === null || _a === void 0 ? void 0 : _a.lifeMinutes) !== null && _b !== void 0 ? _b : this.defaultCacheLifeMinutes;
        const expiration = moment(this.cache.lastUpdate)
            .add(lifeMinutes, 'minutes');
        return moment().isAfter(expiration);
    }
    //#endregion
    getUsers(includePhoto = false, includeDisabled = false, parentOU) {
        var _a, _b, _c;
        // If caching is enabled, there will be two calls to the API if the cache is null/expired
        // One call is to update the cache and the other is to return the data that the user requested
        // Subsequent calls will bypass the API and returned cached data until it expires
        const observable$ = this.http.get(`${this.config.apiUrl}/${includePhoto}/${includeDisabled}/${parentOU !== null && parentOU !== void 0 ? parentOU : ''}`);
        if ((_a = this.config.caching) === null || _a === void 0 ? void 0 : _a.enabled) {
            if (this.cacheExpired) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    var _a, _b, _c;
                    this.cache = {
                        data,
                        lastUpdate: moment()
                    };
                    if ((_a = this.config.logging) === null || _a === void 0 ? void 0 : _a.dataCached) {
                        const lifeMinutes = (_c = (_b = this.config.caching) === null || _b === void 0 ? void 0 : _b.lifeMinutes) !== null && _c !== void 0 ? _c : this.defaultCacheLifeMinutes;
                        console.log(`Data cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if ((_b = this.config.logging) === null || _b === void 0 ? void 0 : _b.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.cache.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if ((_c = this.config.logging) === null || _c === void 0 ? void 0 : _c.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
    }
    getUser(accountName, includePhoto = false) {
        return this.http.get(`${this.config.apiUrl}/name/${accountName}/${includePhoto}`);
    }
    clearCache() {
        var _a;
        this.cache = null;
        if ((_a = this.config.logging) === null || _a === void 0 ? void 0 : _a.cacheCleared) {
            console.log('Cache cleared');
        }
    }
}
ActiveDirectoryService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryService, deps: [{ token: ServiceConfig, optional: true }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
ActiveDirectoryService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () {
        return [{ type: ServiceConfig, decorators: [{
                        type: Optional
                    }] }, { type: i2.HttpClient }];
    } });

class ActiveDirectoryModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ActiveDirectoryModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(config) {
        return {
            ngModule: ActiveDirectoryModule,
            providers: [
                ActiveDirectoryService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
}
ActiveDirectoryModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryModule, deps: [{ token: ActiveDirectoryModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
ActiveDirectoryModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryModule, imports: [CommonModule] });
ActiveDirectoryModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: ActiveDirectoryModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ]
                }]
        }], ctorParameters: function () {
        return [{ type: ActiveDirectoryModule, decorators: [{
                        type: Optional
                    }, {
                        type: SkipSelf
                    }] }];
    } });

/**
 * Generated bundle index. Do not edit.
 */

export { ActiveDirectoryModule, ActiveDirectoryService, ServiceConfig };
//# sourceMappingURL=cleavelandprice-ngx-lib-active-directory.mjs.map
