import * as i0 from '@angular/core';
import { EventEmitter, Injectable, Optional, Output, Component, Inject, Input, NgModule, SkipSelf } from '@angular/core';
import { ConfirmationComponent } from '@cleavelandprice/ngx-lib/dialog';
import * as i4 from '@angular/forms';
import { UntypedFormGroup, UntypedFormControl, Validators, ReactiveFormsModule } from '@angular/forms';
import * as i1 from '@angular/material/dialog';
import { MAT_DIALOG_DATA, MatDialogModule } from '@angular/material/dialog';
import { JwtHelperService } from '@auth0/angular-jwt';
import * as i2 from '@angular/common/http';
import * as i3 from '@cleavelandprice/ngx-lib/sharepoint';
import * as i3$1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i5 from '@angular/material/button';
import { MatButtonModule } from '@angular/material/button';
import * as i6 from '@angular/material/form-field';
import { MatFormFieldModule } from '@angular/material/form-field';
import * as i7 from '@angular/material/icon';
import { MatIconModule } from '@angular/material/icon';
import * as i8 from '@angular/material/input';
import { MatInputModule } from '@angular/material/input';
import * as i9 from '@angular/material/progress-spinner';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';

class ServiceConfig {
}

const defaultConfig = {
    apiUrl: 'http://utilityapi/authenticate/token'
};

//#region Imports
//#endregion
class AuthenticationService {
    //#endregion
    //#region Lifecycle
    // If a ServiceConfig object is not provided when the module is imported, then default values are pulled in from default-config.ts.
    constructor(config, http, sharePointService) {
        this.config = config;
        this.http = http;
        this.sharePointService = sharePointService;
        //#region Fields
        this.authenticatedChanged = new EventEmitter();
        this.authenticatingChanged = new EventEmitter();
        this.authenticationError = new EventEmitter();
        // Flag set to true while the service is attempting to authenticate the user
        this.authenticating = false;
        //Flag set to true when the user is successfully authenticated
        this.authenticated = false;
        this.config = this.config ?? defaultConfig;
        this.checkForExistingToken();
    }
    //#endregion
    //#region Utilities
    //Authenticate the user and store the token in Local Storage
    login(username, password, adminGroup) {
        const today = new Date();
        //const expiration = new Date().setFullYear(today.getFullYear() + 10);
        const request = {
            username: username,
            password: password,
            adminGroup: adminGroup
            //expirationDate: expiration // moment().add(10, 'years').toDate()
        };
        this.authenticating = true;
        this.authenticatingChanged.emit(this.authenticating);
        // Sending a 'raw' POST request so that we can tell HttpClient that we are expecting text instead of Json
        // Otherwise, it would throw an error trying to decode Json from the response
        this.http.request('POST', this.config.apiUrl, { body: request, responseType: 'text' })
            .subscribe({
            next: (token) => this.validateUser(token),
            error: (err) => {
                //console.log('Authentication error', err);
                this.authenticating = false;
                this.authenticatingChanged.emit(this.authenticating);
                this.authenticationError.emit(err);
            }
        });
    }
    //Logout the user and delete the token from Local Storage
    logout() {
        if (localStorage.getItem('token')) {
            localStorage.removeItem('token');
        }
        this.authenticatedUser = null;
        this.authenticated = false;
        this.authenticatedChanged.emit(false);
    }
    validateUser(token) {
        if (!token) {
            return;
        }
        const jwtHelper = new JwtHelperService();
        const decoded = jwtHelper.decodeToken(token);
        if (jwtHelper.isTokenExpired(token)) {
            this.logout();
            return;
        }
        this.authenticatedUser = {
            admin: decoded.role === 'Admin',
            displayName: decoded.displayName,
            accountName: decoded.winaccountname,
            distinguishedName: decoded.distinguishedName,
            employeeNumber: decoded.employeeNumber,
            memberOf: decoded.memberOf
        };
        localStorage.setItem('token', token);
        this.getPhoto();
        this.authenticated = true;
        this.authenticatedChanged.emit(this.authenticated);
        this.authenticating = false;
        this.authenticatingChanged.emit(this.authenticating);
    }
    getPhoto() {
        this.sharePointService.getPhotos()
            .subscribe(photos => {
            // The user may have been logged out by the time the photos came back
            if (!this.authenticatedUser?.employeeNumber) {
                return;
            }
            const photo = photos.find(pic => pic.employeeNumber === +this.authenticatedUser.employeeNumber);
            if (photo) {
                this.authenticatedUser.photoUrl = photo.encodedAbsUrl;
            }
        });
    }
    checkForExistingToken() {
        this.validateUser(localStorage.getItem('token'));
    }
}
AuthenticationService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationService, deps: [{ token: ServiceConfig, optional: true }, { token: i2.HttpClient }, { token: i3.SharepointService }], target: i0.ɵɵFactoryTarget.Injectable });
AuthenticationService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiceConfig, decorators: [{
                    type: Optional
                }] }, { type: i2.HttpClient }, { type: i3.SharepointService }]; }, propDecorators: { authenticatedChanged: [{
                type: Output
            }], authenticatingChanged: [{
                type: Output
            }], authenticationError: [{
                type: Output
            }] } });

class LoginFormComponent {
    constructor(data, dialogRef, authenticationService) {
        this.dialogRef = dialogRef;
        this.authenticationService = authenticationService;
        this.errorMessage = null;
        this.form = new UntypedFormGroup({
            username: new UntypedFormControl(null, [Validators.required]),
            password: new UntypedFormControl(null, [Validators.required]),
            adminGroup: new UntypedFormControl()
        });
        this.form.get('adminGroup')?.setValue(data.adminGroup);
    }
    ngOnInit() {
        this.authenticatedChangedSubscription = this.authenticationService.authenticatedChanged
            .subscribe(this.authChangedHandler.bind(this));
        this.authenticationErrorSubscription = this.authenticationService.authenticationError
            .subscribe(this.errorHandler.bind(this));
    }
    ngOnDestroy() {
        this.authenticationErrorSubscription.unsubscribe();
        this.authenticatedChangedSubscription.unsubscribe();
    }
    authenticate() {
        this.errorMessage = null;
        const { username, password, adminGroup } = this.form.value;
        this.authenticationService.login(username, password, adminGroup);
    }
    authChangedHandler(authenticated) {
        if (authenticated) {
            this.dialogRef.close();
        }
    }
    errorHandler(err) {
        console.log('Authentication error', err);
        this.errorMessage = `${err.name}: ${err.statusText}`;
    }
}
LoginFormComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: LoginFormComponent, deps: [{ token: MAT_DIALOG_DATA }, { token: i1.MatDialogRef }, { token: AuthenticationService }], target: i0.ɵɵFactoryTarget.Component });
LoginFormComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.0", type: LoginFormComponent, selector: "authentication-login-form", ngImport: i0, template: "<h2 mat-dialog-title>\n    <span *ngIf=\"!authenticationService.authenticating\">Login</span>\n    <span *ngIf=\"authenticationService.authenticating\">Authenticating...</span>\n</h2>\n\n<mat-dialog-content>\n    <div id=\"spinner\" *ngIf=\"authenticationService.authenticating\">\n        <mat-progress-spinner color=\"primary\" mode=\"indeterminate\"></mat-progress-spinner>\n    </div>\n\n    <form *ngIf=\"!authenticationService.authenticating\" [formGroup]=\"form\">\n        <mat-form-field>\n            <input matInput formControlName=\"username\" placeholder=\"Username\" (keyup.enter)=\"authenticate()\">\n        </mat-form-field>\n\n        <mat-form-field>\n            <input matInput formControlName=\"password\" type=\"password\" placeholder=\"Password\" (keyup.enter)=\"authenticate()\">\n        </mat-form-field>\n    </form>\n\n    <div *ngIf=\"!authenticationService.authenticating && errorMessage\" id=\"error\">\n        {{ errorMessage }}\n    </div>\n</mat-dialog-content>\n\n<mat-dialog-actions *ngIf=\"!authenticationService.authenticating\">\n    <button mat-raised-button color=\"primary\" (click)=\"authenticate()\"\n        [disabled]=\"form.invalid\">\n        <mat-icon fontIcon=\"done\"></mat-icon>\n        <span>OK</span>\n    </button>\n\n    <button mat-raised-button (click)=\"dialogRef.close()\">\n        <mat-icon fontIcon=\"close\"></mat-icon>\n        <span>Cancel</span>\n    </button>\n</mat-dialog-actions>", styles: ["#spinner{width:100%;height:100%;display:flex;justify-content:center;align-items:center}form{display:grid;gap:1rem}mat-dialog-actions{display:grid;justify-content:center;grid-template-columns:auto auto;gap:1rem}#error{color:red}\n", ".material-icons{font-family:Material Icons;font-weight:400;font-style:normal;font-size:24px;display:inline-block;line-height:1;text-transform:none;letter-spacing:normal;word-wrap:normal;white-space:nowrap;direction:ltr;-webkit-font-smoothing:antialiased;text-rendering:optimizeLegibility;-moz-osx-font-smoothing:grayscale;font-feature-settings:\"liga\"}\n"], dependencies: [{ kind: "directive", type: i3$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i4.ɵNgNoValidate, selector: "form:not([ngNoForm]):not([ngNativeValidate])" }, { kind: "directive", type: i4.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i4.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i4.NgControlStatusGroup, selector: "[formGroupName],[formArrayName],[ngModelGroup],[formGroup],form:not([ngNoForm]),[ngForm]" }, { kind: "directive", type: i4.FormGroupDirective, selector: "[formGroup]", inputs: ["formGroup"], outputs: ["ngSubmit"], exportAs: ["ngForm"] }, { kind: "directive", type: i4.FormControlName, selector: "[formControlName]", inputs: ["formControlName", "disabled", "ngModel"], outputs: ["ngModelChange"] }, { kind: "component", type: i5.MatButton, selector: "button[mat-button], button[mat-raised-button], button[mat-icon-button],             button[mat-fab], button[mat-mini-fab], button[mat-stroked-button],             button[mat-flat-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "directive", type: i1.MatDialogTitle, selector: "[mat-dialog-title], [matDialogTitle]", inputs: ["id"], exportAs: ["matDialogTitle"] }, { kind: "directive", type: i1.MatDialogContent, selector: "[mat-dialog-content], mat-dialog-content, [matDialogContent]" }, { kind: "directive", type: i1.MatDialogActions, selector: "[mat-dialog-actions], mat-dialog-actions, [matDialogActions]", inputs: ["align"] }, { kind: "component", type: i6.MatFormField, selector: "mat-form-field", inputs: ["color", "appearance", "hideRequiredMarker", "hintLabel", "floatLabel"], exportAs: ["matFormField"] }, { kind: "component", type: i7.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "directive", type: i8.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i9.MatProgressSpinner, selector: "mat-progress-spinner, mat-spinner", inputs: ["color", "diameter", "strokeWidth", "mode", "value"], exportAs: ["matProgressSpinner"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: LoginFormComponent, decorators: [{
            type: Component,
            args: [{ selector: 'authentication-login-form', template: "<h2 mat-dialog-title>\n    <span *ngIf=\"!authenticationService.authenticating\">Login</span>\n    <span *ngIf=\"authenticationService.authenticating\">Authenticating...</span>\n</h2>\n\n<mat-dialog-content>\n    <div id=\"spinner\" *ngIf=\"authenticationService.authenticating\">\n        <mat-progress-spinner color=\"primary\" mode=\"indeterminate\"></mat-progress-spinner>\n    </div>\n\n    <form *ngIf=\"!authenticationService.authenticating\" [formGroup]=\"form\">\n        <mat-form-field>\n            <input matInput formControlName=\"username\" placeholder=\"Username\" (keyup.enter)=\"authenticate()\">\n        </mat-form-field>\n\n        <mat-form-field>\n            <input matInput formControlName=\"password\" type=\"password\" placeholder=\"Password\" (keyup.enter)=\"authenticate()\">\n        </mat-form-field>\n    </form>\n\n    <div *ngIf=\"!authenticationService.authenticating && errorMessage\" id=\"error\">\n        {{ errorMessage }}\n    </div>\n</mat-dialog-content>\n\n<mat-dialog-actions *ngIf=\"!authenticationService.authenticating\">\n    <button mat-raised-button color=\"primary\" (click)=\"authenticate()\"\n        [disabled]=\"form.invalid\">\n        <mat-icon fontIcon=\"done\"></mat-icon>\n        <span>OK</span>\n    </button>\n\n    <button mat-raised-button (click)=\"dialogRef.close()\">\n        <mat-icon fontIcon=\"close\"></mat-icon>\n        <span>Cancel</span>\n    </button>\n</mat-dialog-actions>", styles: ["#spinner{width:100%;height:100%;display:flex;justify-content:center;align-items:center}form{display:grid;gap:1rem}mat-dialog-actions{display:grid;justify-content:center;grid-template-columns:auto auto;gap:1rem}#error{color:red}\n", ".material-icons{font-family:Material Icons;font-weight:400;font-style:normal;font-size:24px;display:inline-block;line-height:1;text-transform:none;letter-spacing:normal;word-wrap:normal;white-space:nowrap;direction:ltr;-webkit-font-smoothing:antialiased;text-rendering:optimizeLegibility;-moz-osx-font-smoothing:grayscale;font-feature-settings:\"liga\"}\n"] }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [MAT_DIALOG_DATA]
                }] }, { type: i1.MatDialogRef }, { type: AuthenticationService }]; } });

class LoginComponent {
    constructor(dialog, authenticationService) {
        this.dialog = dialog;
        this.authenticationService = authenticationService;
    }
    login() {
        this.dialog.open(LoginFormComponent, {
            data: { adminGroup: this.adminGroup }
        });
    }
    confirmLogout() {
        this.dialog.open(ConfirmationComponent, {
            data: { message: `Are you sure you want to logout?` }
        })
            .afterClosed()
            .subscribe(result => {
            if (result) {
                this.authenticationService.logout();
            }
        });
    }
}
LoginComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: LoginComponent, deps: [{ token: i1.MatDialog }, { token: AuthenticationService }], target: i0.ɵɵFactoryTarget.Component });
LoginComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.0", type: LoginComponent, selector: "authentication-login", inputs: { adminGroup: "adminGroup" }, ngImport: i0, template: "<div id=\"user-id\">\n    <!-- Logged out -->\n    <mat-icon *ngIf=\"!authenticationService.authenticated\"\n        fontIcon=\"account_circle\"\n        matTooltip=\"Login\"\n        (click)=\"login()\">\n    </mat-icon>\n\n    <!-- Logged in -->\n    <ng-container *ngIf=\"authenticationService.authenticated\">\n        <!-- With photo -->\n        <img *ngIf=\"authenticationService.authenticatedUser?.photoUrl\"\n            id=\"authenticated-user-image\"\n            [src]=\"authenticationService.authenticatedUser?.photoUrl\"\n            matTooltip=\"Logout\"\n            (click)=\"confirmLogout()\" />\n    \n        <!-- Without photo -->\n        <mat-icon *ngIf=\"!authenticationService.authenticatedUser?.photoUrl\"\n            fontIcon=\"account_circle\"\n            matTooltip=\"Logout\"\n            (click)=\"confirmLogout()\">\n        </mat-icon>\n    \n        <!-- Name -->\n        <div id=\"user-name\">\n            {{ authenticationService.authenticatedUser?.displayName }}\n        </div>\n    </ng-container>\n</div>", styles: ["#user-id{display:flex;height:2em;padding:.3em;font-family:Verdana,Geneva,Tahoma,sans-serif}mat-icon{font-size:2em!important;height:2em!important;width:2em!important}img{border-radius:100%;width:2em;height:2em}#user-id>div{display:flex;align-items:center;height:100%;margin-left:.75em;font-size:1.15em}\n", ".material-icons{font-family:Material Icons;font-weight:400;font-style:normal;font-size:24px;display:inline-block;line-height:1;text-transform:none;letter-spacing:normal;word-wrap:normal;white-space:nowrap;direction:ltr;-webkit-font-smoothing:antialiased;text-rendering:optimizeLegibility;-moz-osx-font-smoothing:grayscale;font-feature-settings:\"liga\"}\n"], dependencies: [{ kind: "directive", type: i3$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "component", type: i7.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: LoginComponent, decorators: [{
            type: Component,
            args: [{ selector: 'authentication-login', template: "<div id=\"user-id\">\n    <!-- Logged out -->\n    <mat-icon *ngIf=\"!authenticationService.authenticated\"\n        fontIcon=\"account_circle\"\n        matTooltip=\"Login\"\n        (click)=\"login()\">\n    </mat-icon>\n\n    <!-- Logged in -->\n    <ng-container *ngIf=\"authenticationService.authenticated\">\n        <!-- With photo -->\n        <img *ngIf=\"authenticationService.authenticatedUser?.photoUrl\"\n            id=\"authenticated-user-image\"\n            [src]=\"authenticationService.authenticatedUser?.photoUrl\"\n            matTooltip=\"Logout\"\n            (click)=\"confirmLogout()\" />\n    \n        <!-- Without photo -->\n        <mat-icon *ngIf=\"!authenticationService.authenticatedUser?.photoUrl\"\n            fontIcon=\"account_circle\"\n            matTooltip=\"Logout\"\n            (click)=\"confirmLogout()\">\n        </mat-icon>\n    \n        <!-- Name -->\n        <div id=\"user-name\">\n            {{ authenticationService.authenticatedUser?.displayName }}\n        </div>\n    </ng-container>\n</div>", styles: ["#user-id{display:flex;height:2em;padding:.3em;font-family:Verdana,Geneva,Tahoma,sans-serif}mat-icon{font-size:2em!important;height:2em!important;width:2em!important}img{border-radius:100%;width:2em;height:2em}#user-id>div{display:flex;align-items:center;height:100%;margin-left:.75em;font-size:1.15em}\n", ".material-icons{font-family:Material Icons;font-weight:400;font-style:normal;font-size:24px;display:inline-block;line-height:1;text-transform:none;letter-spacing:normal;word-wrap:normal;white-space:nowrap;direction:ltr;-webkit-font-smoothing:antialiased;text-rendering:optimizeLegibility;-moz-osx-font-smoothing:grayscale;font-feature-settings:\"liga\"}\n"] }]
        }], ctorParameters: function () { return [{ type: i1.MatDialog }, { type: AuthenticationService }]; }, propDecorators: { adminGroup: [{
                type: Input
            }] } });

class AuthenticationModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('AuthenticationModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(config) {
        return {
            ngModule: AuthenticationModule,
            providers: [
                AuthenticationService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
}
AuthenticationModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationModule, deps: [{ token: AuthenticationModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
AuthenticationModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationModule, declarations: [LoginComponent,
        LoginFormComponent], imports: [CommonModule,
        ReactiveFormsModule,
        MatButtonModule,
        MatDialogModule,
        MatFormFieldModule,
        MatIconModule,
        MatInputModule,
        MatProgressSpinnerModule], exports: [LoginComponent,
        LoginFormComponent] });
AuthenticationModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationModule, imports: [CommonModule,
        ReactiveFormsModule,
        MatButtonModule,
        MatDialogModule,
        MatFormFieldModule,
        MatIconModule,
        MatInputModule,
        MatProgressSpinnerModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: AuthenticationModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        LoginComponent,
                        LoginFormComponent
                    ],
                    imports: [
                        CommonModule,
                        ReactiveFormsModule,
                        MatButtonModule,
                        MatDialogModule,
                        MatFormFieldModule,
                        MatIconModule,
                        MatInputModule,
                        MatProgressSpinnerModule
                    ],
                    exports: [
                        LoginComponent,
                        LoginFormComponent
                    ]
                }]
        }], ctorParameters: function () { return [{ type: AuthenticationModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { AuthenticationModule, AuthenticationService, LoginComponent, LoginFormComponent, ServiceConfig };
//# sourceMappingURL=cleavelandprice-ngx-lib-authentication.mjs.map
